/*
@Author - Vishal (http://SqlAndMe.com)
@Contact - Vishal@SqlAndMe.com 

Usage:

EXEC [dbo].[sp_CompareDBSchema]
@sourceDatabaseName = 'AdventureWorks2008R2',
@applyChanges = 0
GO

-- @applyChanges = 1 will execute changescript on target database, use with caution!.
*/ 
CREATE PROCEDURE [dbo].[sp_CompareDBSchema] 
( 
    @sourceDatabaseName sysname , 
    @applyChanges int = 0 
) 
AS 
SET NOCOUNT ON 
DECLARE @SqlVersion int 
SELECT @sqlVersion = cast ( substring ( @@VERSION , 22 , 4 ) AS int ) 

DECLARE @executeScript nvarchar ( max ) 
SET @executeScript = '' 

CREATE TABLE [#ChangeScript] ( [ChangeOrder] smallint NULL , [ChangeScript] nvarchar ( max ) NULL ) ; 
CREATE TABLE [#SourceSchemas] ( [SchemaName] sysname NULL , [Authorization] sysname NULL ) ; 
CREATE TABLE [#TargetSchemas] ( [SchemaName] sysname NULL , [Authorization] sysname NULL ) ; 
CREATE TABLE [#SourceTypes] ( [TypeName] sysname NULL , [BaseType] sysname NULL ) ; 
CREATE TABLE [#TargetTypes] ( [TypeName] sysname NULL , [BaseType] sysname NULL ) ; 
CREATE TABLE [#SourceFKeys] 
( [DatabaseName] sysname NULL , [SchemaName] sysname NULL , 
    [TableName] sysname NULL , [ColumnName] sysname NULL , 
    [ConstraintName] sysname NULL , [ReferencedSchema] sysname NULL , 
    [ReferencedTable] sysname NULL , [ReferencedColumn] sysname NULL , 
    [DeleteAction] sysname NULL , [UpdateAction] sysname NULL ) 
; 
CREATE TABLE [#TargetFKeys] 
( [DatabaseName] sysname NULL , [SchemaName] sysname NULL , 
    [TableName] sysname NULL , [ColumnName] sysname NULL , 
    [ConstraintName] sysname NULL , [ReferencedSchema] sysname NULL , 
    [ReferencedTable] sysname NULL , [ReferencedColumn] sysname NULL , 
    [DeleteAction] sysname NULL , [UpdateAction] sysname NULL ) 
; 
CREATE TABLE [#SourceSchema] 
( [DatabaseName] sysname NULL , [SchemaName] sysname NULL , 
    [TableName] sysname NULL , [ColumnName] sysname NULL , 
    [DataType] sysname NULL , [IsNullable] varchar ( 8 ) NULL , 
    [ComputedColumn] nvarchar ( max ) NULL , [IdentityColumn] nvarchar ( max ) NULL , 
    [DefaultConstraint] sysname NULL , [DCDefinition] nvarchar ( max ) NULL , 
    [CheckConstraint] sysname NULL , [CCDefinition] nvarchar ( max ) NULL , 
    [FileGroup] sysname NULL ) 
; 
CREATE TABLE [#TargetSchema] 
( [DatabaseName] sysname NULL , [SchemaName] sysname NULL , 
    [TableName] sysname NULL , [ColumnName] sysname NULL , 
    [DataType] sysname NULL , [IsNullable] varchar ( 8 ) NULL , 
    [ComputedColumn] nvarchar ( max ) NULL , [IdentityColumn] nvarchar ( max ) NULL , 
    [DefaultConstraint] sysname NULL , [DCDefinition] nvarchar ( max ) NULL , 
    [CheckConstraint] sysname NULL , [CCDefinition] nvarchar ( max ) NULL ) 
; 
CREATE TABLE [#SourceIndexes] 
( [DatabaseName] sysname NULL , [SchemaName] sysname NULL , 
    [TableName] sysname NULL , [IndexName] sysname NULL , 
    [IndexType] nvarchar ( 60 ) NULL , [IsUnique] varchar ( 3 ) NULL , 
    [IsPrimaryKey] varchar ( 3 ) NULL , [IsUniqueKey] varchar ( 3 ) NULL , 
    [IsPadded] varchar ( 25 ) NULL , [IgnoreDupKey] varchar ( 25 ) NULL , 
    [AllowRowLocks] varchar ( 25 ) NULL , [AllowPageLocks] varchar ( 25 ) NULL , 
    [FillFactor] tinyint NULL , [KeyColumns] nvarchar ( max ) NULL , 
    [IncludedColumns] nvarchar ( max ) NULL , [Filter] nvarchar ( max ) NULL , [FileGroupName] sysname NULL ) 
; 
CREATE TABLE [#TargetIndexes] 
( [DatabaseName] sysname NULL , [SchemaName] sysname NULL , [TableName] sysname NULL , [IndexName] sysname NULL , 
    [IndexType] nvarchar ( 60 ) NULL , [IsUnique] varchar ( 3 ) NULL , [IsPrimaryKey] varchar ( 3 ) NULL , [IsUniqueKey] varchar ( 3 ) NULL , 
    [IsPadded] varchar ( 25 ) NULL , [IgnoreDupKey] varchar ( 25 ) NULL , [AllowRowLocks] varchar ( 25 ) NULL , [AllowPageLocks] varchar ( 25 ) NULL , 
    [FillFactor] tinyint NULL , [KeyColumns] nvarchar ( max ) NULL , [IncludedColumns] nvarchar ( max ) NULL , [Filter] nvarchar ( max ) NULL , [FileGroupName] sysname NULL ) 
; 
CREATE TABLE [#IndexedColumn] 
( [TableName] sysname NULL , [IndexName] sysname NULL , 
    [ColumnName] sysname NULL , [IsPrimaryKey] tinyint NULL , 
    [IsUniqueKey] tinyint NULL ) 
; 
CREATE TABLE [#SourceXMLIndexes] 
( [DatabaseName] sysname NULL , [SchemaName] sysname NULL , 
    [TableName] sysname NULL , [IndexName] sysname NULL , 
    [IsPadded] varchar ( 25 ) NULL , [IgnoreDupKey] varchar ( 25 ) NULL , 
    [AllowRowLocks] varchar ( 25 ) NULL , [AllowPageLocks] varchar ( 25 ) NULL , 
    [FillFactor] tinyint NULL , [KeyColumns] nvarchar ( max ) NULL , 
    [PrimaryIndex] nvarchar ( max ) NULL , [SecondaryType] nvarchar ( max ) NULL , 
    [FileGroupName] sysname NULL ) 
; 
CREATE TABLE [#TargetXMLIndexes] 
( [DatabaseName] sysname NULL , [SchemaName] sysname NULL , [TableName] sysname NULL , [IndexName] sysname NULL , [IsPadded] varchar ( 25 ) NULL , [IgnoreDupKey] varchar ( 25 ) NULL , 
    [AllowRowLocks] varchar ( 25 ) NULL , [AllowPageLocks] varchar ( 25 ) NULL , [FillFactor] tinyint NULL , [KeyColumns] nvarchar ( max ) NULL , 
    [PrimaryIndex] nvarchar ( max ) NULL , [SecondaryType] nvarchar ( max ) NULL , [FileGroupName] sysname NULL ) 
; 
CREATE TABLE [#ViewDependencies] ( [object_id] int NULL , [referenced_major_id] int NULL ) ; 

SET @executeScript = 
'INSERT INTO [#SourceTypes]
SELECT QUOTENAME(SCHEMA_NAME(TYP.schema_id)) + ''.'' + QUOTENAME(TYP.name),
CASE WHEN SYSTYP.name IN (''CHAR'', ''VARCHAR'') THEN CASE WHEN TYP.max_length > 0 THEN UPPER(SYSTYP.name) + ''('' + CAST(TYP.max_length AS VARCHAR) + '')''
ELSE UPPER(TYP.name) + ''(MAX)'' END WHEN SYSTYP.name IN (''NCHAR'', ''NVARCHAR'') THEN CASE WHEN TYP.max_length > 0 THEN UPPER(SYSTYP.name) + ''('' + CAST((TYP.max_length/2) AS VARCHAR) + '')''
ELSE UPPER(TYP.name) + ''(MAX)'' END WHEN SYSTYP.name IN (''DECIMAL'', ''NUMERIC'') THEN UPPER(SYSTYP.name) + ''('' + CAST(TYP.precision AS VARCHAR)+ '','' +
CAST(TYP.scale AS VARCHAR) + '')'' ELSE UPPER(SYSTYP.name) END + '' '' +  CASE WHEN TYP.is_nullable = 0 THEN ''NOT NULL'' ELSE ''NULL'' END
FROM ' + @sourceDatabaseName + '.sys.types TYP INNER JOIN ' + @sourceDatabaseName + '.sys.types SYSTYP ON TYP.system_type_id = SYSTYP.system_type_id
WHERE TYP.is_user_defined = 1 AND  SYSTYP.is_user_defined = 0 AND  SYSTYP.name <> ''SYSNAME''' 

EXEC ( 'USE ' + @sourceDatabaseName + ';' + @executeScript ) 

INSERT INTO [#TargetTypes] 
SELECT 
quotename ( SCHEMA_NAME ( TYP.schema_id ) ) + '.' + quotename ( TYP.name ) , 
CASE 
    WHEN SYSTYP.name IN ( 'CHAR' , 'VARCHAR' ) THEN CASE 
    WHEN TYP.max_length > 0 THEN upper ( SYSTYP.name ) + '(' + cast ( TYP.max_length AS varchar ) + ')' 
    ELSE upper ( TYP.name ) + '(MAX)' 
END 
WHEN SYSTYP.name IN ( 'NCHAR' , 'NVARCHAR' ) THEN 
CASE 
    WHEN TYP.max_length > 0 THEN upper ( SYSTYP.name ) + '(' + cast ( ( TYP.max_length / 2 ) AS varchar ) + ')' 
    ELSE upper ( TYP.name ) + '(MAX)' 
END 
WHEN SYSTYP.name IN ( 'DECIMAL' , 'NUMERIC' ) THEN upper ( SYSTYP.name ) + '(' + cast ( TYP.precision AS varchar ) + ',' + cast ( TYP.scale AS varchar ) + ')' 

ELSE upper ( SYSTYP.name ) 
END + ' ' + 
CASE 
    WHEN TYP.is_nullable = 0 THEN 'NOT NULL' 
    ELSE 'NULL' 
END 
FROM sys.types TYP INNER JOIN sys.types SYSTYP ON TYP.system_type_id = SYSTYP.system_type_id WHERE TYP.is_user_defined = 1 AND SYSTYP.is_user_defined = 0 AND SYSTYP.name <> 'SYSNAME' 

SET @executeScript = 
'INSERT INTO [#SourceSchemas]
SELECT S.name, P.name
FROM ' + @sourceDatabaseName + '.sys.schemas S
INNER JOIN ' + @sourceDatabaseName + '.sys.database_principals P ON
S.principal_id = P.principal_id' 

EXEC ( 'USE ' + @sourceDatabaseName + ';' + @executeScript ) 

INSERT INTO [#TargetSchemas] 
SELECT 
S.name , 
P.name 
FROM sys.schemas S 
INNER JOIN sys.database_principals P ON S.principal_id = P.principal_id 

SET @executeScript = 
'INSERT INTO [#SourceSchema]
SELECT ''' + @sourceDatabaseName + ''', OBJECT_SCHEMA_NAME(TBL.object_id), TBL.name, COL.name, 
CASE WHEN TYP.name IN (''CHAR'', ''VARCHAR'') THEN CASE WHEN COL.max_length > 0 THEN UPPER(TYP.name) + ''('' + CAST(COL.max_length AS VARCHAR) + '')'' ELSE UPPER(TYP.name) + ''(MAX)'' END
WHEN TYP.name IN (''NCHAR'', ''NVARCHAR'') THEN CASE WHEN COL.max_length > 0 THEN UPPER(TYP.name) + ''('' + CAST((COL.max_length/2) AS VARCHAR) + '')'' ELSE UPPER(TYP.name) + ''(MAX)'' END
WHEN TYP.name IN (''DECIMAL'', ''NUMERIC'') THEN UPPER(TYP.name) + ''('' + CAST(COL.precision AS VARCHAR)+ '','' + CAST(COL.scale AS VARCHAR) + '')'' ELSE UPPER(TYP.name) END,
CASE WHEN COL.is_nullable = 0 THEN ''NOT NULL'' ELSE ''NULL'' END, CASE WHEN COL.is_computed = 1 THEN CASE WHEN CCOL.is_persisted = 1 THEN CCOL.definition + '' PERSISTED''
ELSE CCOL.definition END ELSE NULL END, CASE WHEN COL.is_identity = 1 THEN ''IDENTITY ('' + CAST(ICOL.seed_value AS VARCHAR) + '', '' +
CAST(ICOL.increment_value AS VARCHAR) + '')'' END, DC.name, DC.definition, CC.name, CC.definition, FG.name
FROM ' + @sourceDatabaseName + '.sys.tables TBL
INNER JOIN ' + @sourceDatabaseName + '.sys.columns COL ON TBL.object_id = COL.object_id
INNER JOIN ' + @sourceDatabaseName + '.sys.types TYP ON COL.user_type_id = TYP.user_type_id
INNER JOIN ' + @sourceDatabaseName + '.sys.indexes IX ON TBL.object_id = IX.object_id AND IX.index_id <= 1
INNER JOIN ' + @sourceDatabaseName + '.sys.filegroups FG ON IX.data_space_id = FG.data_space_id
LEFT JOIN  ' + @sourceDatabaseName + '.sys.computed_columns CCOL ON COL.object_id = CCOL.object_id AND COL.column_id = CCOL.column_id
LEFT JOIN  ' + @sourceDatabaseName + '.sys.identity_columns ICOL ON COL.object_id = ICOL.object_id
LEFT JOIN  ' + @sourceDatabaseName + '.sys.default_constraints DC ON TBL.object_id = DC.parent_object_id AND COL.column_id = DC.parent_column_id
LEFT JOIN  ' + @sourceDatabaseName + '.sys.check_constraints CC ON TBL.object_id = CC.parent_object_id AND COL.column_id = CC.parent_column_id' 

EXEC ( 'USE ' + @sourceDatabaseName + ';' + @executeScript ) 

INSERT INTO [#TargetSchema] 
SELECT 
db_name ( ) , 
OBJECT_SCHEMA_NAME ( TBL.object_id ) , 
TBL.name , 
COL.name , 
CASE 
    WHEN TYP.name IN ( 'CHAR' , 'VARCHAR' ) THEN CASE 
    WHEN COL.max_length > 0 THEN upper ( TYP.name ) + '(' + cast ( COL.max_length AS varchar ) + ')' 
    ELSE upper ( TYP.name ) + '(MAX)' 
END 
WHEN TYP.name IN ( 'NCHAR' , 'NVARCHAR' ) THEN 
CASE 
    WHEN COL.max_length > 0 THEN upper ( TYP.name ) + '(' + cast ( ( COL.max_length / 2 ) AS varchar ) + ')' 
    ELSE upper ( TYP.name ) + '(MAX)' 
END 
WHEN TYP.name IN ( 'DECIMAL' , 'NUMERIC' ) THEN 
upper ( TYP.name ) + '(' + cast ( COL.precision AS varchar ) + ',' + cast ( COL.scale AS varchar ) + ')' 

ELSE upper ( TYP.name ) 
END 
, 
CASE 
    WHEN COL.is_nullable = 0 THEN 'NOT NULL' 
    ELSE 'NULL' 
END 
, 
CASE 
    WHEN COL.is_computed = 1 THEN CASE 
    WHEN CCOL.is_persisted = 1 THEN CCOL.definition + ' PERSISTED' 
    ELSE CCOL.definition 
END 
ELSE NULL 
END 
, 
CASE 
    WHEN COL.is_identity = 1 THEN 'IDENTITY (' + cast ( ICOL.seed_value AS varchar ) + ', ' + cast ( ICOL.increment_value AS varchar ) + ')' 
END 
, DC.name , DC.definition , CC.name , CC.definition 
FROM sys.tables TBL 
INNER JOIN sys.columns COL ON TBL.object_id = COL.object_id 
INNER JOIN sys.types TYP ON COL.user_type_id = TYP.user_type_id 
LEFT JOIN sys.computed_columns CCOL ON COL.object_id = CCOL.object_id AND COL.column_id = CCOL.column_id 
LEFT JOIN sys.identity_columns ICOL ON COL.object_id = ICOL.object_id 
LEFT JOIN sys.default_constraints DC ON TBL.object_id = DC.parent_object_id AND COL.column_id = DC.parent_column_id 
LEFT JOIN sys.check_constraints CC ON TBL.object_id = CC.parent_object_id AND COL.column_id = CC.parent_column_id 

SET @executeScript = 
'INSERT INTO [#SourceFKeys]
SELECT DISTINCT ''' + @sourceDatabaseName + ''', OBJECT_SCHEMA_NAME(FC.parent_object_id),  OBJECT_NAME(FC.parent_object_id),
STUFF((SELECT '', '' + QUOTENAME(COL_NAME(FCC1.parent_object_id, FCC1.parent_column_id)) FROM sys.foreign_key_columns FCC1 WHERE FCC1.constraint_object_id = FC.object_id
FOR XML PATH('''')), 1, 1, ''''), FC.name, OBJECT_SCHEMA_NAME(FCC.referenced_object_id), OBJECT_NAME(FCC.referenced_object_id),
STUFF((SELECT '', '' + QUOTENAME(COL_NAME(FCC1.referenced_object_id, FCC1.referenced_column_id)) FROM sys.foreign_key_columns FCC1 WHERE FCC1.constraint_object_id = FC.object_id FOR XML PATH('''')), 1, 1, ''''),
''ON DELETE '' + CASE WHEN FC.delete_referential_action = 1 THEN ''CASCADE'' WHEN FC.delete_referential_action = 2 THEN ''SET NULL'' WHEN FC.delete_referential_action = 3 THEN ''SET DEFAULT'' ELSE NULL END,
''ON UPDATE '' + CASE WHEN FC.update_referential_action = 1 THEN ''CASCADE'' WHEN FC.update_referential_action = 2 THEN ''SET NULL'' WHEN FC.update_referential_action = 3 THEN ''SET DEFAULT'' ELSE NULL END
FROM ' + @sourceDatabaseName + '.sys.foreign_keys FC INNER JOIN ' + @sourceDatabaseName + '.sys.foreign_key_columns FCC ON FC.object_id = FCC.constraint_object_id' 

EXEC ( 'USE ' + @sourceDatabaseName + ';' + @executeScript ) 

INSERT INTO [#TargetFKeys] 
SELECT DISTINCT 
db_name ( ) , 
OBJECT_SCHEMA_NAME ( FC.parent_object_id ) , 
object_name ( FC.parent_object_id ) , 
stuff 
( 
    ( 
        SELECT ', ' + quotename ( col_name ( FCC1.parent_object_id , FCC1.parent_column_id ) ) FROM sys.foreign_key_columns FCC1 WHERE FCC1.constraint_object_id = FC.object_id FOR XML PATH ( '' ) 
    ) , 1 , 1 , '' 
) , 
FC.name , 
OBJECT_SCHEMA_NAME ( FCC.referenced_object_id ) , 
object_name ( FCC.referenced_object_id ) , 
stuff 
( 
    ( 
        SELECT ', ' + quotename ( col_name ( FCC1.referenced_object_id , FCC1.referenced_column_id ) ) FROM sys.foreign_key_columns FCC1 WHERE FCC1.constraint_object_id = FC.object_id FOR XML PATH ( '' ) 
    ) , 1 , 1 , '' 
) , 
'ON DELETE ' + 
CASE 
    WHEN FC.delete_referential_action = 1 THEN 'CASCADE' 
    WHEN FC.delete_referential_action = 2 THEN 'SET NULL' 
    WHEN FC.delete_referential_action = 3 THEN 'SET DEFAULT' 
    ELSE NULL 
END 
, 
'ON UPDATE ' + 
CASE 
    WHEN FC.update_referential_action = 1 THEN 'CASCADE' 
    WHEN FC.update_referential_action = 2 THEN 'SET NULL' 
    WHEN FC.update_referential_action = 3 THEN 'SET DEFAULT' 
    ELSE NULL 
END 
FROM sys.foreign_keys FC 
INNER JOIN sys.foreign_key_columns FCC ON FC.object_id = FCC.constraint_object_id 

IF ( @sqlVersion = 2005 ) 
BEGIN 
    SET @executeScript = 
    'INSERT INTO [#SourceIndexes]
    (DatabaseName, SchemaName, TableName, IndexName, IndexType, IsUnique, IsPrimaryKey, IsUniqueKey, IsPadded,
    IgnoreDupKey, AllowRowLocks, AllowPageLocks, [FillFactor], KeyColumns, IncludedColumns, FileGroupName)
    SELECT ''' + @sourceDatabaseName + ''', SCHEMA_NAME(TBL.schema_id), OBJECT_NAME(TBL.object_id),
    IDX.name, IDX.type_desc, CASE WHEN IDX.is_unique = 1 THEN ''Yes'' ELSE ''No'' END,
    CASE WHEN IDX.is_primary_key = 1 THEN ''Yes'' ELSE ''No'' END, CASE WHEN IDX.is_unique_constraint = 1 THEN ''Yes'' ELSE ''No'' END,
    ''PAD_INDEX  = '' + CASE WHEN IDX.is_padded = 1 THEN ''ON'' ELSE ''OFF'' END,
    ''IGNORE_DUP_KEY = '' + CASE WHEN IDX.ignore_dup_key = 1 THEN ''ON'' ELSE ''OFF'' END, ''ALLOW_ROW_LOCKS  = '' + CASE WHEN IDX.allow_row_locks = 1 THEN ''ON'' ELSE ''OFF'' END,
    ''ALLOW_PAGE_LOCKS  = '' + CASE WHEN IDX.allow_page_locks = 1 THEN ''ON'' ELSE ''OFF'' END, CASE WHEN IDX.fill_factor = 0 THEN 100 ELSE IDX.fill_factor END,
    LEFT(IndexedColumns, LEN(IndexedColumns) -1), LEFT(IncludedColumns, LEN(IncludedColumns) -1), FG.name
    FROM ' + @sourceDatabaseName + '.sys.tables TBL
    INNER JOIN ' + @sourceDatabaseName + '.sys.indexes IDX ON TBL.object_id = IDX.object_id
    LEFT JOIN   ' + @sourceDatabaseName + '.sys.filegroups FG ON IDX.data_space_id = FG.data_space_id
    CROSS APPLY
    ( SELECT QUOTENAME(COLS.Name) + CASE WHEN is_descending_key = 1 THEN '' DESC'' ELSE '' ASC'' END + '', ''
    FROM ' + @sourceDatabaseName + '.sys.index_columns IXCL
    INNER JOIN ' + @sourceDatabaseName + '.sys.columns COLS ON IXCL.object_id = COLS.object_id AND IXCL.column_id = COLS.column_id
    WHERE IDX.object_id = IXCL.object_id AND  IXCL.is_included_column = 0 AND  IDX.index_id = IXCL.index_id ORDER BY IXCL.Key_Ordinal FOR XML PATH('''')
    ) Indexed (IndexedColumns)
    CROSS APPLY
    ( SELECT QUOTENAME(COLS2.Name) + '', '' FROM ' + @sourceDatabaseName + '.sys.index_columns ICLS
    INNER JOIN ' + @sourceDatabaseName + '.sys.columns COLS2 ON ICLS.object_id = COLS2.object_id AND ICLS.column_id = COLS2.column_id
    WHERE IDX.object_id = ICLS.object_id AND  ICLS.is_included_column = 1 AND  IDX.index_id = ICLS.index_id ORDER BY ICLS.Key_Ordinal FOR XML PATH('''')
    ) Included (IncludedColumns) WHERE IDX.type_desc <> ''HEAP''' 
END 
ELSE 
BEGIN 
    SET @executeScript = 
    'INSERT INTO [#SourceIndexes]
    (DatabaseName, SchemaName, TableName, IndexName, IndexType, IsUnique, IsPrimaryKey, IsUniqueKey, IsPadded,
    IgnoreDupKey, AllowRowLocks, AllowPageLocks, [FillFactor], KeyColumns, IncludedColumns, Filter, FileGroupName)
    SELECT ''' + @sourceDatabaseName + ''', SCHEMA_NAME(TBL.schema_id), OBJECT_NAME(TBL.object_id), IDX.name, IDX.type_desc,
    CASE WHEN IDX.is_unique = 1 THEN ''Yes'' ELSE ''No'' END, CASE WHEN IDX.is_primary_key = 1 THEN ''Yes'' ELSE ''No'' END, CASE WHEN IDX.is_unique_constraint = 1 THEN ''Yes'' ELSE ''No'' END,
    ''PAD_INDEX  = '' + CASE WHEN IDX.is_padded = 1 THEN ''ON'' ELSE ''OFF'' END, ''IGNORE_DUP_KEY = '' + CASE WHEN IDX.ignore_dup_key = 1 THEN ''ON'' ELSE ''OFF'' END, ''ALLOW_ROW_LOCKS  = '' + CASE WHEN IDX.allow_row_locks = 1 THEN ''ON'' ELSE ''OFF'' END, ''ALLOW_PAGE_LOCKS  = '' + CASE WHEN IDX.allow_page_locks = 1 THEN ''ON'' ELSE ''OFF'' END, CASE WHEN IDX.fill_factor = 0 THEN 100 ELSE IDX.fill_factor END, LEFT(IndexedColumns, LEN(IndexedColumns) -1), LEFT(IncludedColumns, LEN(IncludedColumns) -1), IDX.filter_definition, FG.name
    FROM ' + @sourceDatabaseName + '.sys.tables TBL
    INNER JOIN ' + @sourceDatabaseName + '.sys.indexes IDX ON TBL.object_id = IDX.object_id
    LEFT JOIN   ' + @sourceDatabaseName + '.sys.filegroups FG ON IDX.data_space_id = FG.data_space_id
    CROSS APPLY
    ( SELECT QUOTENAME(COLS.Name) + CASE WHEN is_descending_key = 1 THEN '' DESC'' ELSE '' ASC'' END + '', ''
    FROM ' + @sourceDatabaseName + '.sys.index_columns IXCL INNER JOIN ' + @sourceDatabaseName + '.sys.columns COLS
    ON IXCL.object_id = COLS.object_id AND IXCL.column_id = COLS.column_id WHERE IDX.object_id = IXCL.object_id
    AND  IXCL.is_included_column = 0 AND  IDX.index_id = IXCL.index_id ORDER BY IXCL.Key_Ordinal FOR XML PATH('''')
    ) Indexed (IndexedColumns)
    CROSS APPLY
    ( SELECT QUOTENAME(COLS2.Name) + '', '' FROM ' + @sourceDatabaseName + '.sys.index_columns ICLS INNER JOIN ' + @sourceDatabaseName + '.sys.columns COLS2 ON ICLS.object_id = COLS2.object_id AND ICLS.column_id = COLS2.column_id
    WHERE IDX.object_id = ICLS.object_id AND  ICLS.is_included_column = 1 AND  IDX.index_id = ICLS.index_id ORDER BY ICLS.Key_Ordinal FOR XML PATH('''')
    ) Included (IncludedColumns)
    WHERE IDX.type_desc <> ''HEAP'' AND  IDX.type_desc <> ''XML''' 
END 

EXEC ( 'USE ' + @sourceDatabaseName + ';' + @executeScript ) 

IF ( @sqlVersion = 2005 ) 
BEGIN 
    INSERT INTO [#TargetIndexes] 
    ( DatabaseName , SchemaName , TableName , IndexName , IndexType , IsUnique , IsPrimaryKey , IsUniqueKey , IsPadded , 
        IgnoreDupKey , AllowRowLocks , AllowPageLocks , [FillFactor] , KeyColumns , IncludedColumns , FileGroupName ) 
    SELECT 
    db_name ( ) , 
    SCHEMA_NAME ( TBL.schema_id ) , 
    object_name ( TBL.object_id ) , 
    IDX.name , 
    IDX.type_desc , 
    CASE 
        WHEN IDX.is_unique = 1 THEN 'Yes' 
        ELSE 'No' 
    END 
    , 
    CASE 
        WHEN IDX.is_primary_key = 1 THEN 'Yes' 
        ELSE 'No' 
    END 
    , 
    CASE 
        WHEN IDX.is_unique_constraint = 1 THEN 'Yes' 
        ELSE 'No' 
    END 
    , 
    'PAD_INDEX  = ' + 
    CASE 
        WHEN IDX.is_padded = 1 THEN 'ON' 
        ELSE 'OFF' 
    END 
    , 
    'IGNORE_DUP_KEY = ' + 
    CASE 
        WHEN IDX.ignore_dup_key = 1 THEN 'ON' 
        ELSE 'OFF' 
    END 
    , 
    'ALLOW_ROW_LOCKS  = ' + 
    CASE 
        WHEN IDX.allow_row_locks = 1 THEN 'ON' 
        ELSE 'OFF' 
    END 
    , 
    'ALLOW_PAGE_LOCKS  = ' + 
    CASE 
        WHEN IDX.allow_page_locks = 1 THEN 'ON' 
        ELSE 'OFF' 
    END 
    , 
    CASE 
        WHEN IDX.fill_factor = 0 THEN 100 
        ELSE IDX.fill_factor 
    END 
    , 
    left ( IndexedColumns , len ( IndexedColumns ) - 1 ) , 
    left ( IncludedColumns , len ( IncludedColumns ) - 1 ) , 
    FG.name 
    FROM sys.tables TBL 
    INNER JOIN sys.indexes IDX ON TBL.object_id = IDX.object_id 
    LEFT JOIN sys.filegroups FG ON IDX.data_space_id = FG.data_space_id CROSS APPLY 
    ( 
        SELECT quotename ( COLS.Name ) + 
        CASE 
            WHEN is_descending_key = 1 THEN ' DESC' 
            ELSE ' ASC' 
        END + ', ' FROM sys.index_columns IXCL 
        INNER JOIN sys.columns COLS ON IXCL.object_id = COLS.object_id 
        AND IXCL.column_id = COLS.column_id 
        WHERE IDX.object_id = IXCL.object_id 
        AND IXCL.is_included_column = 0 
        AND IDX.index_id = IXCL.index_id 
        ORDER BY IXCL.Key_Ordinal FOR XML PATH ( '' ) 
    ) 
    Indexed ( IndexedColumns ) 
    CROSS APPLY 
    ( 
        SELECT quotename ( COLS2.Name ) + ', ' FROM sys.index_columns ICLS 
        INNER JOIN sys.columns COLS2 ON ICLS.object_id = COLS2.object_id 
        AND ICLS.column_id = COLS2.column_id 
        WHERE IDX.object_id = ICLS.object_id 
        AND ICLS.is_included_column = 1 
        AND IDX.index_id = ICLS.index_id 
        ORDER BY ICLS.Key_Ordinal FOR XML PATH ( '' ) 
    ) 
    Included ( IncludedColumns ) 
    WHERE IDX.type_desc <> 'HEAP' 
    AND IDX.type_desc <> 'XML' 

END 
ELSE 
BEGIN 
    INSERT INTO [#TargetIndexes] 
    ( DatabaseName , SchemaName , TableName , IndexName , IndexType , IsUnique , IsPrimaryKey , IsUniqueKey , IsPadded , 
        IgnoreDupKey , AllowRowLocks , AllowPageLocks , [FillFactor] , KeyColumns , IncludedColumns , Filter , FileGroupName ) 
    SELECT 
    db_name ( ) , 
    SCHEMA_NAME ( TBL.schema_id ) , 
    object_name ( TBL.object_id ) , 
    IDX.name , 
    IDX.type_desc , 
    CASE 
        WHEN IDX.is_unique = 1 THEN 'Yes' 
        ELSE 'No' 
    END 
    , 
    CASE 
        WHEN IDX.is_primary_key = 1 THEN 'Yes' 
        ELSE 'No' 
    END 
    , 
    CASE 
        WHEN IDX.is_unique_constraint = 1 THEN 'Yes' 
        ELSE 'No' 
    END 
    , 
    'PAD_INDEX  = ' + 
    CASE 
        WHEN IDX.is_padded = 1 THEN 'ON' 
        ELSE 'OFF' 
    END 
    , 
    'IGNORE_DUP_KEY = ' + 
    CASE 
        WHEN IDX.ignore_dup_key = 1 THEN 'ON' 
        ELSE 'OFF' 
    END 
    , 
    'ALLOW_ROW_LOCKS  = ' + 
    CASE 
        WHEN IDX.allow_row_locks = 1 THEN 'ON' 
        ELSE 'OFF' 
    END 
    , 
    'ALLOW_PAGE_LOCKS  = ' + 
    CASE 
        WHEN IDX.allow_page_locks = 1 THEN 'ON' 
        ELSE 'OFF' 
    END 
    , 
    CASE 
        WHEN IDX.fill_factor = 0 THEN 100 
        ELSE IDX.fill_factor 
    END 
    , 
    left ( IndexedColumns , len ( IndexedColumns ) - 1 ) , 
    left ( IncludedColumns , len ( IncludedColumns ) - 1 ) , 
    IDX.filter_definition , 
    FG.name 
    FROM sys.tables TBL 
    INNER JOIN sys.indexes IDX ON TBL.object_id = IDX.object_id 
    LEFT JOIN sys.filegroups FG ON IDX.data_space_id = FG.data_space_id CROSS APPLY 
    ( 
        SELECT quotename ( COLS.Name ) + 
        CASE 
            WHEN is_descending_key = 1 THEN ' DESC' 
            ELSE ' ASC' 
        END + ', ' FROM sys.index_columns IXCL 
        INNER JOIN sys.columns COLS ON IXCL.object_id = COLS.object_id 
        AND IXCL.column_id = COLS.column_id 
        WHERE IDX.object_id = IXCL.object_id 
        AND IXCL.is_included_column = 0 
        AND IDX.index_id = IXCL.index_id 
        ORDER BY IXCL.Key_Ordinal FOR XML PATH ( '' ) 
    ) 
    Indexed ( IndexedColumns ) 
    CROSS APPLY 
    ( 
        SELECT quotename ( COLS2.Name ) + ', ' FROM sys.index_columns ICLS 
        INNER JOIN sys.columns COLS2 ON ICLS.object_id = COLS2.object_id 
        AND ICLS.column_id = COLS2.column_id 
        WHERE IDX.object_id = ICLS.object_id 
        AND ICLS.is_included_column = 1 
        AND IDX.index_id = ICLS.index_id 
        ORDER BY ICLS.Key_Ordinal FOR XML PATH ( '' ) 
    ) 
    Included ( IncludedColumns ) WHERE IDX.type_desc <> 'HEAP' 

END 

INSERT INTO [#IndexedColumn] 
SELECT 
TBL.name , 
IDX.name , 
COL.name , 
IDX.is_primary_key , 
IDX.is_unique_constraint 
FROM sys.tables TBL 
INNER JOIN sys.index_columns IDXCLS ON TBL.object_id = IDXCLS.object_id 
INNER JOIN sys.indexes IDX ON IDXCLS.index_id = IDX.index_id 
AND IDX.object_id = TBL.object_id 
INNER JOIN sys.columns COL ON IDXCLS.column_id = COL.column_id 
AND COL.object_id = TBL.object_id 

SET @executeScript = 
'
INSERT INTO [#SourceXMLIndexes]
SELECT DB_NAME(), SCHEMA_NAME(TBL.schema_id), OBJECT_NAME(TBL.object_id),
IDX.name, ''PAD_INDEX  = '' + CASE WHEN IDX.is_padded = 1 THEN ''ON'' ELSE ''OFF'' END, ''IGNORE_DUP_KEY = '' + CASE WHEN IDX.ignore_dup_key = 1 THEN ''ON'' ELSE ''OFF'' END,
''ALLOW_ROW_LOCKS  = '' + CASE WHEN IDX.allow_row_locks = 1 THEN ''ON'' ELSE ''OFF'' END, ''ALLOW_PAGE_LOCKS  = '' + CASE WHEN IDX.allow_page_locks = 1 THEN ''ON'' ELSE ''OFF'' END,
CASE WHEN IDX.fill_factor = 0 THEN 100 ELSE IDX.fill_factor END, LEFT(IndexedColumns, LEN(IndexedColumns) -1), IDX1.name, ''FOR '' + IDX.secondary_type_desc, FG.name
FROM ' + @sourceDatabaseName + '.sys.tables TBL INNER JOIN ' + @sourceDatabaseName + '.sys.xml_indexes IDX ON TBL.object_id = IDX.object_id
LEFT  JOIN ' + @sourceDatabaseName + '.sys.xml_indexes IDX1 ON IDX.using_xml_index_id = IDX1.index_id AND IDX.object_id = IDX1.object_id
LEFT JOIN   ' + @sourceDatabaseName + '.sys.filegroups FG ON IDX.data_space_id = FG.data_space_id
CROSS APPLY ( SELECT QUOTENAME(COLS.Name) + '',''
FROM ' + @sourceDatabaseName + '.sys.index_columns IXCL INNER JOIN ' + @sourceDatabaseName + '.sys.columns COLS ON IXCL.object_id = COLS.object_id AND IXCL.column_id = COLS.column_id
WHERE IDX.object_id = IXCL.object_id AND  IXCL.is_included_column = 0 AND  IDX.index_id = IXCL.index_id ORDER BY IXCL.Key_Ordinal FOR XML PATH('''') ) Indexed (IndexedColumns) WHERE IDX.type_desc = ''XML''' 

EXEC ( 'USE ' + @sourceDatabaseName + ';' + @executeScript ) 

INSERT INTO #TargetXMLIndexes 
SELECT 
db_name ( ) , 
SCHEMA_NAME ( TBL.schema_id ) , 
object_name ( TBL.object_id ) , 
IDX.name , 
'PAD_INDEX  = ' + 
CASE 
    WHEN IDX.is_padded = 1 THEN 'ON' 
    ELSE 'OFF' 
END 
, 
'IGNORE_DUP_KEY = ' + 
CASE 
    WHEN IDX.ignore_dup_key = 1 THEN 'ON' 
    ELSE 'OFF' 
END 
, 
'ALLOW_ROW_LOCKS  = ' + 
CASE 
    WHEN IDX.allow_row_locks = 1 THEN 'ON' 
    ELSE 'OFF' 
END 
, 
'ALLOW_PAGE_LOCKS  = ' + 
CASE 
    WHEN IDX.allow_page_locks = 1 THEN 'ON' 
    ELSE 'OFF' 
END 
, 
CASE 
    WHEN IDX.fill_factor = 0 THEN 100 
    ELSE IDX.fill_factor 
END 
, 
left ( IndexedColumns , len ( IndexedColumns ) - 1 ) , 
IDX1.name , 
'FOR ' + IDX.secondary_type_desc , 
FG.name 
FROM sys.tables TBL 
INNER JOIN sys.xml_indexes IDX ON TBL.object_id = IDX.object_id 
LEFT JOIN sys.xml_indexes IDX1 ON IDX.using_xml_index_id = IDX1.index_id 
AND IDX.object_id = IDX1.object_id 
LEFT JOIN sys.filegroups FG ON IDX.data_space_id = FG.data_space_id CROSS APPLY 
( 
    SELECT quotename ( COLS.Name ) + ',' FROM sys.index_columns IXCL 
    INNER JOIN sys.columns COLS ON IXCL.object_id = COLS.object_id 
    AND IXCL.column_id = COLS.column_id 
    WHERE IDX.object_id = IXCL.object_id 
    AND IXCL.is_included_column = 0 
    AND IDX.index_id = IXCL.index_id 
    ORDER BY IXCL.Key_Ordinal FOR XML PATH ( '' ) 
) 
Indexed ( IndexedColumns ) 
WHERE IDX.type_desc = 'XML' 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
140 , 
CASE 
    WHEN T.IndexName IS NULL THEN 'CREATE PRIMARY XML INDEX ' + quotename ( S.IndexName ) + ' ON ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + char ( 13 ) + ' ( ' + S.KeyColumns + ' ) ' + char ( 13 ) + 'WITH (' + S.IsPadded + ', SORT_IN_TEMPDB = OFF, ' + S.IgnoreDupKey + ', ' + S.AllowRowLocks + ', ' + S.AllowPageLocks + ')' 
    WHEN S.KeyColumns <> T.KeyColumns THEN 'CREATE PRIMARY XML INDEX ' + quotename ( S.IndexName ) + ' ON ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + char ( 13 ) + ' ( ' + S.KeyColumns + ' ) ' + char ( 13 ) + 'WITH (' + S.IsPadded + ', SORT_IN_TEMPDB = OFF, DROP_EXISTING = ON, ' + S.IgnoreDupKey + ', ' + S.AllowRowLocks + ', ' + S.AllowPageLocks + ')' 
END 
FROM #SourceXMLIndexes S 
LEFT JOIN #TargetXMLIndexes T ON S.SchemaName = T.SchemaName 
AND S.TableName = T.TableName 
AND S.IndexName = T.IndexName 
WHERE S.PrimaryIndex IS NULL 
AND 
( ( T.IndexName IS NULL ) OR ( S.KeyColumns <> T.KeyColumns ) ) 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
145 , 
CASE 
    WHEN T.IndexName IS NULL THEN 'CREATE XML INDEX ' + quotename ( S.IndexName ) + ' ON ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + char ( 13 ) + ' ( ' + S.KeyColumns + ' ) ' + char ( 13 ) + 'USING XML INDEX ' + quotename ( S.PrimaryIndex ) + ' ' + S.SecondaryType + ' ' + char ( 13 ) + 'WITH (' + S.IsPadded + ', SORT_IN_TEMPDB = OFF, ' + S.IgnoreDupKey + ', ' + S.AllowRowLocks + ', ' + S.AllowPageLocks + ')' 
    WHEN S.KeyColumns <> T.KeyColumns THEN 'CREATE XML INDEX ' + quotename ( S.IndexName ) + ' ON ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + char ( 13 ) + ' ( ' + S.KeyColumns + ' ) ' + char ( 13 ) + 'USING XML INDEX ' + quotename ( S.PrimaryIndex ) + ' ' + S.SecondaryType + ' ' + char ( 13 ) + 'WITH (' + S.IsPadded + ', SORT_IN_TEMPDB = OFF, DROP_EXISTING = ON, ' + S.IgnoreDupKey + ', ' + S.AllowRowLocks + ', ' + S.AllowPageLocks + ')' 
END 
FROM #SourceXMLIndexes S 
LEFT JOIN #TargetXMLIndexes T ON S.SchemaName = T.SchemaName 
AND S.TableName = T.TableName 
AND S.IndexName = T.IndexName 
WHERE S.PrimaryIndex IS NOT NULL 
AND 
( ( T.IndexName IS NULL ) OR ( S.KeyColumns <> T.KeyColumns ) ) 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
10 , 
'DROP TYPE ' + S.TypeName 
FROM [#SourceTypes] S 
INNER JOIN [#TargetTypes] T ON S.TypeName = T.TypeName WHERE S.BaseType <> T.BaseType 

DELETE FROM [#TargetTypes] WHERE TypeName IN 
( 
    SELECT S.TypeName FROM [#SourceTypes] S 
    INNER JOIN [#TargetTypes] T ON S.TypeName = T.TypeName WHERE S.BaseType <> T.BaseType 
) 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
15 , 
'CREATE TYPE ' + S.TypeName + ' FROM ' + S.BaseType 
FROM [#SourceTypes] S 
LEFT JOIN [#TargetTypes] T ON S.TypeName = T.TypeName WHERE T.TypeName IS NULL 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
20 , 
'CREATE SCHEMA ' + quotename ( S.SchemaName ) + ' AUTHORIZATION ' + quotename ( S. [Authorization] ) 
FROM #SourceSchemas S 
LEFT JOIN #TargetSchemas T ON S.SchemaName = T.SchemaName WHERE T.SchemaName IS NULL 

SELECT * 
INTO [#SourceSchemaNew] FROM [#SourceSchema] WHERE TableName NOT IN 
( 
    SELECT TableName FROM [#TargetSchema] 
) 

DELETE FROM [#SourceSchema] 
WHERE TableName NOT IN 
( 
    SELECT TableName FROM [#TargetSchema] 
) 

DECLARE @newTableCount int 
SET @newTableCount = 0 
SELECT @newTableCount = count ( DISTINCT SchemaName + TableName ) FROM [#SourceSchemaNew] 

DECLARE @scriptCreateTable nvarchar ( max ) 
DECLARE @newSchemaName sysname 
DECLARE @newTableName sysname 
DECLARE @newTableFileGroup sysname 

WHILE @newTableCount > 0 
BEGIN 
    SET @scriptCreateTable = '' 
    SET @newSchemaName = '' 
    SET @newTableName = '' 
    SET @newTableFileGroup = '' 

    SELECT TOP 1 
    @newSchemaName = SchemaName , 
    @newTableName = TableName , 
    @newTableFileGroup = FileGroup 
    FROM [#SourceSchemaNew] 

    SELECT @scriptCreateTable = @scriptCreateTable + coalesce ( A.CreateScript , '' ) FROM 
    ( 
        SELECT TOP 1 [CreateScript] = 'CREATE TABLE ' + quotename ( SchemaName ) + '.' + quotename ( TableName ) + '(' FROM [#SourceSchemaNew] 
        WHERE SchemaName = @newSchemaName 
        AND TableName = @newTableName 
        UNION ALL 

        SELECT quotename ( ColumnName ) + ' ' + DataType + ' ' + IdentityColumn + ' ' + IsNullable + ', ' + char ( 13 ) FROM [#SourceSchemaNew] 
        WHERE IdentityColumn IS NOT NULL 
        AND SchemaName = @newSchemaName 
        AND TableName = @newTableName 
        UNION ALL 

        SELECT quotename ( ColumnName ) + ' ' + DataType + ' ' + IsNullable + coalesce ( ( ' DEFAULT ' + DCDefinition ) , '' ) + ', ' + char ( 13 ) FROM [#SourceSchemaNew] 
        WHERE IdentityColumn IS NULL 
        AND ComputedColumn IS NULL 
        AND SchemaName = @newSchemaName 
        AND TableName = @newTableName 
        UNION ALL 

        SELECT quotename ( ColumnName ) + ' AS ' + ComputedColumn + ', ' + char ( 13 ) FROM [#SourceSchemaNew] 
        WHERE ComputedColumn IS NOT NULL 
        AND SchemaName = @newSchemaName 
        AND TableName = @newTableName 
    ) 
    A 

    SET @scriptCreateTable = stuff 
    ( @scriptCreateTable , len ( @scriptCreateTable ) - 2 , 2 , ') ON ' + 
        quotename ( @newTableFileGroup ) ) 

    INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
    SELECT 
    25 , 
    @scriptCreateTable 

    INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
    SELECT 
    45 , 
    'ALTER TABLE ' + quotename ( SchemaName ) + '.' + quotename ( TableName ) + '  WITH CHECK ADD  CONSTRAINT ' + quotename ( CheckConstraint ) + ' CHECK  (' + CCDefinition + ')' 
    FROM [#SourceSchemaNew] 
    WHERE SchemaName = @newSchemaName 
    AND TableName = @newTableName 
    AND CCDefinition IS NOT NULL 

    INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
    SELECT 
    95 , 
    'ALTER TABLE ' + quotename ( SchemaName ) + '.' + quotename ( TableName ) + '  WITH CHECK ADD CONSTRAINT ' + quotename ( ConstraintName ) + ' FOREIGN KEY (' + ColumnName + ') REFERENCES ' + quotename ( ReferencedSchema ) + '.' + quotename ( ReferencedTable ) + ' (' + ReferencedColumn + ') ' + coalesce ( UpdateAction , '' ) + ' ' + coalesce ( DeleteAction , '' ) 
    FROM [#SourceFKeys] 
    WHERE SchemaName = quotename ( @newSchemaName ) 
    AND TableName = quotename ( @newTableName ) 

    DELETE FROM [#SourceFKeys] WHERE SchemaName = quotename ( @newSchemaName ) AND TableName = quotename ( @newTableName ) 
    DELETE FROM [#SourceSchemaNew] WHERE SchemaName = @newSchemaName AND TableName = @newTableName 
    SET @newTableCount = @newTableCount - 1 
END 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
30 , 
CASE 
    WHEN S.ComputedColumn IS NOT NULL THEN 'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' ADD ' + quotename ( S.ColumnName ) + ' AS ' + S.ComputedColumn 
    ELSE 'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' ADD ' + quotename ( S.ColumnName ) + ' ' + S.DataType + ' ' + coalesce ( S.IdentityColumn , '' ) + ' ' + S.IsNullable 
END 
FROM [#SourceSchema] S 
LEFT JOIN [#TargetSchema] T ON S.TableName = T.TableName 
AND S.ColumnName = T.ColumnName 
WHERE T.ColumnName IS NULL 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
35 , 
CASE 
    WHEN ( IXCLS.IsPrimaryKey = 1 ) OR ( IXCLS.IsUniqueKey = 1 ) THEN 'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( T.TableName ) + ' DROP CONSTRAINT ' + quotename ( IXCLS.IndexName ) 
    ELSE 'DROP INDEX ' + quotename ( IXCLS.IndexName ) + ' ON ' + quotename ( S.SchemaName ) + '.' + quotename ( T.TableName ) + ' WITH ( ONLINE = OFF )' 
END 
FROM [#SourceSchema] S 
LEFT JOIN [#TargetSchema] T ON S.TableName = T.TableName 
AND S.ColumnName = T.ColumnName 
LEFT JOIN [#IndexedColumn] IXCLS ON S.TableName = IXCLS.TableName 
AND S.ColumnName = IXCLS.ColumnName 
WHERE 
( ( S.DataType <> T.DataType ) OR ( S.IsNullable <> T.IsNullable ) ) 
AND IXCLS.IndexName IS NOT NULL 

SELECT * 
INTO #TempResults FROM [#SourceIndexes] WHERE 1 = 2 

DELETE FROM [#TargetIndexes] 
OUTPUT DELETED.* INTO #TempResults 
WHERE IndexName IN 
( 
    SELECT IXCLS.IndexName FROM [#SourceSchema] S 
    LEFT JOIN [#TargetSchema] T ON S.TableName = T.TableName 
    AND S.ColumnName = T.ColumnName 
    LEFT JOIN [#IndexedColumn] IXCLS ON S.TableName = IXCLS.TableName 
    AND S.ColumnName = IXCLS.ColumnName 
    WHERE ( S.DataType <> T.DataType ) 
    OR 
    ( S.IsNullable <> T.IsNullable ) 
    AND IXCLS.IndexName IS NOT NULL 
) 

IF ( @sqlVersion = 2005 ) 
BEGIN 
    INSERT INTO [#SourceIndexes] 
    ( DatabaseName , SchemaName , TableName , IndexName , IndexType , IsUnique , IsPrimaryKey , IsUniqueKey , IsPadded , 
        IgnoreDupKey , AllowRowLocks , AllowPageLocks , [FillFactor] , KeyColumns , IncludedColumns , FileGroupName ) 
    SELECT 
    @sourceDatabaseName , 
    TR.SchemaName , 
    TR.TableName , 
    TR.IndexName , 
    TR.IndexType , 
    TR.IsUnique , 
    TR.IsPrimaryKey , 
    TR.IsUniqueKey , 
    TR.IsPadded , 
    TR.IgnoreDupKey , 
    TR.AllowRowLocks , 
    TR.AllowPageLocks , 
    TR. [FillFactor] , 
    TR.KeyColumns , 
    TR.IncludedColumns , 
    TR.FileGroupName 
    FROM #TempResults TR 
    LEFT JOIN [#SourceIndexes] SI ON SI.SchemaName = TR.SchemaName 
    AND SI.TableName = TR.TableName 
    AND SI.IndexName = TR.IndexName 
    WHERE SI.IndexName IS NULL 

END 
ELSE 
BEGIN 
    INSERT INTO [#SourceIndexes] 
    ( DatabaseName , SchemaName , TableName , IndexName , IndexType , IsUnique , IsPrimaryKey , IsUniqueKey , IsPadded , 
        IgnoreDupKey , AllowRowLocks , AllowPageLocks , [FillFactor] , KeyColumns , IncludedColumns , Filter , FileGroupName ) 
    SELECT 
    @sourceDatabaseName , 
    TR.SchemaName , 
    TR.TableName , 
    TR.IndexName , 
    TR.IndexType , 
    TR.IsUnique , 
    TR.IsPrimaryKey , 
    TR.IsUniqueKey , 
    TR.IsPadded , 
    TR.IgnoreDupKey , 
    TR.AllowRowLocks , 
    TR.AllowPageLocks , 
    TR. [FillFactor] , 
    TR.KeyColumns , 
    TR.IncludedColumns , 
    TR.Filter , 
    TR.FileGroupName 
    FROM #TempResults TR 
    LEFT JOIN [#SourceIndexes] SI ON SI.SchemaName = TR.SchemaName 
    AND SI.TableName = TR.TableName 
    AND SI.IndexName = TR.IndexName 
    WHERE SI.IndexName IS NULL 

END 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
40 , 
'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' ALTER COLUMN ' + quotename ( S.ColumnName ) + ' ' + S.DataType + ' ' + S.IsNullable 
FROM [#SourceSchema] S 
LEFT JOIN [#TargetSchema] T ON S.TableName = T.TableName 
AND S.ColumnName = T.ColumnName 
WHERE ( S.DataType <> T.DataType ) 
OR 
( S.IsNullable <> T.IsNullable ) 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
45 , 
'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + '  WITH CHECK ADD  CONSTRAINT ' + quotename ( S.CheckConstraint ) + ' CHECK  (' + S.CCDefinition + ')' 
FROM [#SourceSchema] S 
LEFT JOIN [#TargetSchema] T ON S.TableName = T.TableName 
AND S.ColumnName = T.ColumnName 
WHERE T.CheckConstraint IS NULL 
AND S.CheckConstraint IS NOT NULL 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
50 , 
'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' ADD  CONSTRAINT ' + quotename ( S.DefaultConstraint ) + '  DEFAULT ' + S.DCDefinition + ' FOR ' + quotename ( S.ColumnName ) 
FROM [#SourceSchema] S 
LEFT JOIN [#TargetSchema] T ON S.TableName = T.TableName 
AND S.ColumnName = T.ColumnName 
WHERE T.DefaultConstraint IS NULL 
AND S.DefaultConstraint IS NOT NULL 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
50 , 
'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' DROP CONSTRAINT ' + quotename ( T.DefaultConstraint ) + '; ' + 'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + 
' ADD  CONSTRAINT ' + quotename ( S.DefaultConstraint ) + '  DEFAULT ' + S.DCDefinition + ' FOR ' + quotename ( S.ColumnName ) 
FROM [#SourceSchema] S 
LEFT JOIN [#TargetSchema] T ON S.TableName = T.TableName 
AND S.ColumnName = T.ColumnName 
WHERE T.DefaultConstraint IS NOT NULL 
AND S.DefaultConstraint IS NOT NULL 
AND T.DCDefinition <> S.DCDefinition 

DELETE S FROM [#SourceFKeys] S 
INNER JOIN [#TargetFKeys] T ON S.ConstraintName = T.ConstraintName 
WHERE ( S.ReferencedSchema = T.ReferencedSchema ) AND ( S.ReferencedTable = T.ReferencedTable ) AND ( S.ReferencedColumn = T.ReferencedColumn ) 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
85 , 
'EXEC sp_Rename ''' + T.ConstraintName + ''', ''' + S.ConstraintName + '''' 
FROM [#SourceFKeys] S 
INNER JOIN [#TargetFKeys] T ON S.SchemaName = T.SchemaName 
AND S.TableName = T.TableName 
AND S.ColumnName = T.ColumnName 
AND S.ReferencedSchema = T.ReferencedSchema 
AND S.ReferencedTable = T.ReferencedTable 
AND S.ReferencedColumn = T.ReferencedColumn 
WHERE S.ConstraintName <> T.ConstraintName 
AND S.ConstraintName NOT IN 
( 
    SELECT T.ConstraintName FROM [#TargetFKeys] T 
) 

DELETE S FROM [#SourceFKeys] S 
INNER JOIN [#TargetFKeys] T ON S.SchemaName = T.SchemaName AND S.TableName = T.TableName AND S.ColumnName = T.ColumnName 
WHERE ( S.ReferencedSchema = T.ReferencedSchema ) AND ( S.ReferencedTable = T.ReferencedTable ) AND ( S.ReferencedColumn = T.ReferencedColumn ) 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
90 , 
'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' DROP CONSTRAINT ' + quotename ( T.ConstraintName ) 
FROM [#SourceFKeys] S 
INNER JOIN [#TargetFKeys] T ON S.SchemaName = T.SchemaName 
AND S.TableName = T.TableName 
AND S.ColumnName = T.ColumnName 
WHERE ( S.ReferencedSchema <> T.ReferencedSchema ) 
OR 
( S.ReferencedTable <> T.ReferencedTable ) 
OR 
( S.ReferencedColumn <> T.ReferencedColumn ) 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
95 , 
'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + '  WITH CHECK ADD CONSTRAINT ' + quotename ( S.ConstraintName ) + ' FOREIGN KEY (' + S.ColumnName + ') REFERENCES ' + 
quotename ( S.ReferencedSchema ) + '.' + quotename ( S.ReferencedTable ) + ' (' + S.ReferencedColumn + ') ' + coalesce ( S.UpdateAction , '' ) + ' ' + coalesce ( S.DeleteAction , '' ) 
FROM [#SourceFKeys] S 
LEFT JOIN [#TargetFKeys] T ON S.SchemaName = T.SchemaName 
AND S.TableName = T.TableName 
AND S.ColumnName = T.ColumnName 
WHERE ( S.ReferencedSchema <> coalesce ( T.ReferencedSchema , '' ) ) 
OR 
( S.ReferencedTable <> coalesce ( T.ReferencedTable , '' ) ) 
OR 
( S.ReferencedColumn <> coalesce ( T.ReferencedColumn , '' ) ) 

IF ( @sqlVersion = 2005 ) 
BEGIN 
    INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
    SELECT 
    100 , 
    CASE 
        WHEN S.IsUnique = 'No' THEN 'CREATE ' + S.IndexType + ' INDEX ' + quotename ( S.IndexName ) + ' ON ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' ( ' + S.KeyColumns + ' ) WITH (' + S.IsPadded + ', STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, ' + S.IgnoreDupKey + ', DROP_EXISTING = OFF, ONLINE = OFF, ' + S.AllowRowLocks + ', ' + S.AllowPageLocks + ', FILLFACTOR = ' + cast ( S. [FillFactor] AS varchar ( 3 ) ) + ') ON ' + quotename ( S.FileGroupName ) 
        WHEN S.IsUnique = 'Yes' THEN 'CREATE UNIQUE ' + S.IndexType + ' INDEX ' + quotename ( S.IndexName ) + ' ON ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' ( ' + S.KeyColumns + ' ) WITH (' + S.IsPadded + ', STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, ' + S.IgnoreDupKey + ', DROP_EXISTING = OFF, ONLINE = OFF, ' + S.AllowRowLocks + ', ' + S.AllowPageLocks + ', FILLFACTOR = ' + cast ( S. [FillFactor] AS varchar ( 3 ) ) + ') ON ' + quotename ( S.FileGroupName ) 
    END 
    FROM [#SourceIndexes] S 
    LEFT JOIN [#TargetIndexes] T ON S.TableName = T.TableName 
    AND S.IndexName = T.IndexName 
    WHERE S.IsPrimaryKey = 'No' 
    AND S.IsUniqueKey = 'No' 
    AND T.IndexName IS NULL 

END 
ELSE 
BEGIN 
    INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
    SELECT 
    100 , 
    CASE 
        WHEN S.IsUnique = 'No' THEN 'CREATE ' + S.IndexType + ' INDEX ' + quotename ( S.IndexName ) + ' ON ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' ( ' + S.KeyColumns + ' ) ' + coalesce ( 'WHERE ' + S.Filter + ' ' , '' ) + 'WITH (' + S.IsPadded + ', STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, ' + S.IgnoreDupKey + ', DROP_EXISTING = OFF, ONLINE = OFF, ' + S.AllowRowLocks + ', ' + S.AllowPageLocks + ', FILLFACTOR = ' + cast ( S. [FillFactor] AS varchar ( 3 ) ) + ') ON ' + quotename ( S.FileGroupName ) 
        WHEN S.IsUnique = 'Yes' THEN 'CREATE UNIQUE ' + S.IndexType + ' INDEX ' + quotename ( S.IndexName ) + ' ON ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' ( ' + S.KeyColumns + ' ) WITH (' + S.IsPadded + ', STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, ' + S.IgnoreDupKey + ', DROP_EXISTING = OFF, ONLINE = OFF, ' + S.AllowRowLocks + ', ' + S.AllowPageLocks + ', FILLFACTOR = ' + cast ( S. [FillFactor] AS varchar ( 3 ) ) + ') ON ' + quotename ( S.FileGroupName ) 
    END 
    FROM [#SourceIndexes] S 
    LEFT JOIN [#TargetIndexes] T ON S.TableName = T.TableName 
    AND S.IndexName = T.IndexName 
    WHERE S.IsPrimaryKey = 'No' 
    AND S.IsUniqueKey = 'No' 
    AND T.IndexName IS NULL 

END 

IF ( @sqlVersion = 2005 ) 
BEGIN 
    INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
    SELECT 
    100 , 
    'CREATE ' + S.IndexType + ' INDEX ' + quotename ( S.IndexName ) + ' ON ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' ( ' + S.KeyColumns + ') ' + coalesce ( 'INCLUDE ( ' + S.IncludedColumns + ') ' , '' ) + 
    'WITH (' + S.IsPadded + ', STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, ' + S.IgnoreDupKey + ', DROP_EXISTING = ON, ONLINE = OFF, ' + S.AllowRowLocks + ', ' + S.AllowPageLocks + ', FILLFACTOR = ' + cast ( S. [FillFactor] AS varchar ( 3 ) ) + ') ON ' + quotename ( S.FileGroupName ) 
    FROM [#SourceIndexes] S 
    LEFT JOIN [#TargetIndexes] T ON S.TableName = T.TableName 
    AND S.IndexName = T.IndexName 
    WHERE S.IsPrimaryKey = 'No' 
    AND S.IsUniqueKey = 'No' 
    AND S.IndexName = T.IndexName 
    AND 
    ( ( S.KeyColumns <> T.KeyColumns ) OR ( S.IncludedColumns <> T.IncludedColumns ) ) 

END 
ELSE 
BEGIN 
    INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
    SELECT 
    100 , 
    'CREATE ' + S.IndexType + ' INDEX ' + quotename ( S.IndexName ) + ' ON ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' ( ' + S.KeyColumns + ') ' + coalesce ( 'WHERE ' + S.Filter + ' ' , '' ) + 
    coalesce ( 'INCLUDE ( ' + S.IncludedColumns + ') ' , '' ) + 'WITH (' + S.IsPadded + ', STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, ' + S.IgnoreDupKey + ', DROP_EXISTING = ON, ONLINE = OFF, ' + S.AllowRowLocks + ', ' + S.AllowPageLocks + ', FILLFACTOR = ' + cast ( S. [FillFactor] AS varchar ( 3 ) ) + ') ON ' + quotename ( S.FileGroupName ) 
    FROM [#SourceIndexes] S 
    LEFT JOIN [#TargetIndexes] T ON S.TableName = T.TableName 
    AND S.IndexName = T.IndexName 
    WHERE S.IsPrimaryKey = 'No' 
    AND S.IsUniqueKey = 'No' 
    AND S.IndexName = T.IndexName 
    AND 
    ( ( S.KeyColumns <> T.KeyColumns ) OR ( S.IncludedColumns <> T.IncludedColumns ) OR ( S.Filter <> T.Filter ) ) 

END 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
55 , 
'EXEC sp_Rename ''' + T.IndexName + ''', ''' + S.IndexName + '''' 
FROM [#SourceIndexes] S 
LEFT JOIN [#TargetIndexes] T ON S.TableName = T.TableName 
AND S.IsPrimaryKey = T.IsPrimaryKey 
WHERE S.IsPrimaryKey = 'Yes' 
AND 
( ( S.IndexName <> T.IndexName ) AND ( S.KeyColumns = T.KeyColumns ) ) 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
60 , 
'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' DROP CONSTRAINT ' + quotename ( T.IndexName ) 
FROM [#SourceIndexes] S 
LEFT JOIN [#TargetIndexes] T ON S.TableName = T.TableName 
AND S.IsPrimaryKey = T.IsPrimaryKey 
WHERE S.IsPrimaryKey = 'Yes' 
AND 
( T.IndexName IS NOT NULL AND ( S.KeyColumns <> T.KeyColumns ) ) 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
65 , 
'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' ADD CONSTRAINT ' + quotename ( S.IndexName ) + ' PRIMARY KEY ' + S.IndexType + ' ( ' + S.KeyColumns + ' ) WITH (' + S.IsPadded + ', STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, ' + S.IgnoreDupKey + ', ONLINE = OFF, ' + S.AllowRowLocks + ', ' + S.AllowPageLocks + ', FILLFACTOR = ' + cast ( S. [FillFactor] AS varchar ( 3 ) ) + ') ON ' + quotename ( S.FileGroupName ) 
FROM [#SourceIndexes] S 
LEFT JOIN [#TargetIndexes] T ON S.TableName = T.TableName 
AND S.IsPrimaryKey = T.IsPrimaryKey 
WHERE S.IsPrimaryKey = 'Yes' 
AND 
( T.IndexName IS NULL OR ( S.KeyColumns <> T.KeyColumns ) ) 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
70 , 
'EXEC sp_Rename ''' + T.IndexName + ''', ''' + S.IndexName + '''' 
FROM [#SourceIndexes] S 
LEFT JOIN [#TargetIndexes] T ON S.TableName = T.TableName 
AND S.IsUniqueKey = T.IsUniqueKey 
WHERE S.IsUniqueKey = 'Yes' 
AND 
( ( S.IndexName <> T.IndexName ) AND ( S.KeyColumns = T.KeyColumns ) ) 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
75 , 
'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' DROP CONSTRAINT ' + quotename ( T.IndexName ) 
FROM [#SourceIndexes] S 
LEFT JOIN [#TargetIndexes] T ON S.TableName = T.TableName 
AND S.IsUniqueKey = T.IsUniqueKey 
WHERE S.IsUniqueKey = 'Yes' 
AND 
( T.IndexName IS NOT NULL AND ( S.KeyColumns <> T.KeyColumns ) ) 

INSERT INTO [#ChangeScript] ( ChangeOrder , ChangeScript ) 
SELECT 
80 , 
'ALTER TABLE ' + quotename ( S.SchemaName ) + '.' + quotename ( S.TableName ) + ' ADD CONSTRAINT ' + quotename ( S.IndexName ) + ' UNIQUE ' + S.IndexType + ' ( ' + S.KeyColumns + ' ) WITH (' + 
S.IsPadded + ', STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, ' + S.IgnoreDupKey + ', ONLINE = OFF, ' + S.AllowRowLocks + ', ' + S.AllowPageLocks + ', FILLFACTOR = ' + cast ( S. [FillFactor] AS varchar ( 3 ) ) + ') ON ' + quotename ( S.FileGroupName ) 
FROM [#SourceIndexes] S 
LEFT JOIN [#TargetIndexes] T ON S.TableName = T.TableName 
AND S.IsUniqueKey = T.IsUniqueKey 
WHERE S.IsUniqueKey = 'Yes' 
AND 
( T.IndexName IS NULL OR ( S.KeyColumns <> T.KeyColumns ) ) 

SET @executeScript = 
'INSERT INTO [#ViewDependencies]
SELECT DISTINCT object_id, referenced_major_id FROM ' + @sourceDatabaseName + '.sys.sql_dependencies' 

EXEC ( @executeScript ) 

IF EXISTS 
( 
    SELECT * FROM sys.objects 
    WHERE object_id = object_id ( N'[dbo].[sp_CompareDBSchema_ReplaceString]' ) 
    AND type IN ( N'FN' ) 
) 
DROP FUNCTION [dbo].[sp_CompareDBSchema_ReplaceString] ; 

SET @executeScript = 
'CREATE FUNCTION [dbo].[sp_CompareDBSchema_ReplaceString]
(@scriptString NVARCHAR(MAX),@type NVARCHAR(50))
RETURNS NVARCHAR(MAX)
AS 
BEGIN 
DECLARE @createText NVARCHAR(MAX) 
DECLARE @spaceCount INT
DECLARE @maxIterations INT
SET @createText = ''CREATE''
SET @spaceCount = 1
SET @maxIterations = 1000
WHILE (@spaceCount < @maxIterations)
BEGIN
IF (CHARINDEX(@createText + SPACE(@spaceCount) + @type, @scriptString) <> 0)
BEGIN
SET @scriptString = REPLACE(@scriptString, @createText + SPACE(@spaceCount) + @type, ''ALTER'' + SPACE(@spaceCount) + @type)
BREAK;
END
SET @spaceCount = @spaceCount + 1
END
RETURN @scriptString
END' 

EXEC ( @executeScript ) 

SET @executeScript = 
';WITH ViewHierarchy(ViewObjectID, ViewName, ViewLevel) 
AS  (  SELECT VW.object_id, VW.name, 0 AS ViewLevel
FROM ' + @sourceDatabaseName + '.sys.views VW WHERE VW.name NOT IN
( SELECT DISTINCT V.name FROM [#ViewDependencies] D
INNER JOIN ' + @sourceDatabaseName + '.sys.views V ON D.object_id = V.object_id
LEFT JOIN ' + @sourceDatabaseName + '.sys.views V1 ON D.referenced_major_id = V1.object_id WHERE V1.name IS NOT NULL )
UNION ALL SELECT RV.object_id, RV.name, ViewLevel + 1 FROM [#ViewDependencies] D
INNER JOIN ViewHierarchy VH ON D.referenced_major_id = VH.ViewObjectID INNER JOIN ' + @sourceDatabaseName + '.sys.views RV ON D.object_id = RV.object_id )
INSERT INTO [#ChangeScript] (ChangeOrder, ChangeScript)
SELECT [ChangeOrder] =  CASE WHEN C.definition IS NULL THEN CASE WHEN A.type = ''V'' THEN 105 
WHEN A.type = ''P''  THEN 110 WHEN A.type = ''TR'' THEN 115 WHEN A.type = ''FN'' THEN 120 WHEN A.type = ''TF'' THEN 120 
END  WHEN B.definition <> C.definition THEN CASE  WHEN A.type = ''V'' THEN 105 WHEN A.type = ''P''  THEN 130 
WHEN A.type = ''TR'' THEN 135  WHEN A.type = ''FN'' THEN 125   WHEN A.type = ''TF'' THEN 125   END END,
[ChangeScript] = CASE WHEN B.definition <> C.definition THEN 
CASE WHEN A.type = ''V'' THEN [dbo].[sp_CompareDBSchema_ReplaceString](B.definition, ''VIEW'') WHEN A.type = ''P''  THEN [dbo].[sp_CompareDBSchema_ReplaceString](B.definition, ''PROC'')
WHEN A.type = ''TR'' THEN [dbo].[sp_CompareDBSchema_ReplaceString](B.definition, ''TRIGGER'') WHEN A.type = ''FN'' THEN [dbo].[sp_CompareDBSchema_ReplaceString](B.definition, ''FUNCTION'')
WHEN A.type = ''TF'' THEN [dbo].[sp_CompareDBSchema_ReplaceString](B.definition, ''FUNCTION'') END ELSE B.definition END
FROM ' + @sourceDatabaseName + '.sys.objects A
INNER JOIN ' + @sourceDatabaseName + '.sys.sql_modules B ON A.object_id = B.object_id
INNER JOIN ' + @sourceDatabaseName + '.sys.schemas S ON A.schema_id = S.schema_id
LEFT JOIN ( SELECT ViewName, NewLevel = MAX(ViewLevel) FROM ViewHierarchy GROUP BY ViewName ) VH ON A.name = VH.ViewName
LEFT JOIN ( SELECT AA.name, BB.definition FROM sys.objects AA
INNER JOIN sys.sql_modules BB ON AA.object_id = BB.object_id INNER JOIN sys.schemas SS ON AA.schema_id = SS.schema_id WHERE AA.type IN (''V'',''P'',''TR'',''FN'',''TF'') ) C ON A.name = C.Name
WHERE A.type IN (''V'',''P'',''TR'',''FN'',''TF'')
AND (LTRIM(RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(B.definition, '' '', ''''), CHAR(160), ''''), CHAR(13), ''''), CHAR(10), '''')))
<> LTRIM(RTRIM(REPLACE(REPLACE(REPLACE(REPLACE(C.definition, '' '', ''''), CHAR(160), ''''), CHAR(13), ''''), CHAR(10), ''''))) ) OR (C.definition IS NULL) ORDER BY VH.NewLevel' 

EXEC ( @executeScript ) 

IF EXISTS 
( 
    SELECT * FROM sys.objects 
    WHERE object_id = object_id ( N'[dbo].[sp_CompareDBSchema_ReplaceString]' ) 
    AND type IN ( N'FN' ) 
) 
DROP FUNCTION [dbo].[sp_CompareDBSchema_ReplaceString] ; 

IF ( @applyChanges = 1 ) 
BEGIN 
    SET @executeScript = '' ; 
    DECLARE changeScriptCursor CURSOR FOR 
    SELECT ChangeScript FROM #ChangeScript 
    ORDER BY ChangeOrder 

    OPEN changeScriptCursor 
    FETCH NEXT FROM changeScriptCursor INTO @executeScript 
    WHILE @@FETCH_STATUS = 0 
    BEGIN 
        BEGIN 
            TRY 
            PRINT ( @executeScript ) 

            EXEC ( @executeScript ) 
        END 
        TRY 
        BEGIN 
            CATCH 
            DECLARE @errorMsg varchar ( max ) 
            SELECT @errorMsg = 'Error : ' + cast ( ERROR_NUMBER ( ) AS varchar ) + ', ' + cast ( ERROR_SEVERITY ( ) AS varchar ) + ', ' + cast ( ERROR_STATE ( ) AS varchar ) + ', ' + 
            ERROR_PROCEDURE ( ) + ', ' + cast ( ERROR_LINE ( ) AS varchar ) + ', ' + ERROR_MESSAGE ( ) 

            PRINT ( @errorMsg ) 

        END 
        CATCH 
        FETCH NEXT FROM changeScriptCursor INTO @executeScript 
    END 
    CLOSE changeScriptCursor 
    DEALLOCATE changeScriptCursor 
END 
ELSE 
BEGIN 
    SELECT 
    ChangeOrder , 
    ChangeScript 
    FROM [#ChangeScript] 
    ORDER BY ChangeOrder 

END 
GO