
USE tempdb;
GO
SET NOCOUNT ON;

--Create a test table
CREATE TABLE #BinaryCheckSumDemo 
      (Id INT IDENTITY (1,1),
       EmployeeName NVARCHAR(50),
       CityName NVARCHAR(50),
       BinaryCheckSum AS BINARY_CHECKSUM(EmployeeName, CityName)
      );
GO

--Insert some test data
INSERT INTO #BinaryCheckSumDemo 
       (EmployeeName, CityName)
VALUES ('John','Boston'),
       ('Tim','Paris'),
       ('Robert','New York'),
       ('Jim','Orlando');
GO

SELECT * FROM #BinaryCheckSumDemo;
GO


--Try changing the case of the value and
--see if the BINARY_CHECKSUM value changes
UPDATE #BinaryCheckSumDemo
SET CityName = UPPER(CityName)
WHERE EmployeeName = 'Tim';
GO

SELECT * FROM #BinaryCheckSumDemo;
GO


USE tempdb;
GO

--Create a test table to study collation sensitivity of CHECKSUM
CREATE TABLE #BinaryChecksumCollationSensivity 
      (SourceKey INT IDENTITY(1,1),
       CaseInsensitiveColumn VARCHAR(20) 
                             COLLATE SQL_Latin1_General_CP1_CI_AS,
       CaseSensitiveColumn VARCHAR(20)
                           COLLATE SQL_Latin1_General_CP1_CS_AS,
       BinaryChecksumInsensitive AS BINARY_CHECKSUM(CaseInsensitiveColumn),
       BinaryChecksumSensitive AS BINARY_CHECKSUM(CaseSensitiveColumn)
      );
GO

--Insert some test data, make sure that both values have the same data
INSERT INTO #BinaryChecksumCollationSensivity
       (CaseInsensitiveColumn, CaseSensitiveColumn)
VALUES ('Microsoft', 'Microsoft' ),
       ('Windows', 'Windows' ), 
       ('Sql Server', 'Sql Server'),
       (NULL, NULL ),
       ('', '' );
GO

--Fetch the data from the test table
SELECT * FROM #BinaryChecksumCollationSensivity;
GO

USE tempdb;
GO
SET NOCOUNT ON;

--Symmetric values fail CHECKSUM test
SELECT BINARY_CHECKSUM('Nakul') AS 'Before Change',
       BINARY_CHECKSUM('NANANANANANANANANakul') AS 'After Change';
GO

USE tempdb;
GO
SET NOCOUNT ON;

--Create a test table
CREATE TABLE #BinaryCheckSumDemo 
      (Id INT IDENTITY (1,1),
       EmployeeName NVARCHAR(50),
       CityName NVARCHAR(50),
       DataBinaryCheckSum1 AS BINARY_CHECKSUM(EmployeeName, CityName),
       DataBinaryCheckSum2 AS BINARY_CHECKSUM(CityName, EmployeeName)
      );
GO

--Insert some test data
INSERT INTO #BinaryCheckSumDemo
       (EmployeeName, CityName)
VALUES ('John','Boston'),
       ('Tim','London'),
       ('Robert','New York'),
       ('Jim','Orlando');
GO

SELECT * FROM #BinaryCheckSumDemo;
GO

USE tempdb;
GO
SET NOCOUNT ON;

--Symmetric values fail CHECKSUM test
SELECT HASHBYTES('MD5','Nakul') AS 'Before Change',
       HASHBYTES('MD5','NANANANANANANANANakul') AS 'After Change';
GO

USE tempdb;
GO
SET NOCOUNT ON;

--Create a test table
CREATE TABLE CheckSumDemo 
       (Id INT IDENTITY (1,1),
        EmployeeName NVARCHAR(50),
        CityName NVARCHAR(50),
        DataVersion ROWVERSION,
        DataCheckSum AS CHECKSUM(EmployeeName, CityName),
        DataBinaryCheckSum AS BINARY_CHECKSUM(EmployeeName, CityName),
        DataHashBytes AS HASHBYTES('MD5', (EmployeeName + CityName))
       );
GO

--Check for associated data-types
SELECT OBJECT_NAME(sc.object_id),
       sc.name AS ColumnName,
       st.name AS Datatype,
       sc.max_length AS ColumnLength,
       sc.collation_name AS Collation,
       sc.is_computed AS IsColumnComputed
FROM sys.columns AS sc 
LEFT OUTER JOIN sys.types AS st ON sc.user_type_id = st.user_type_id
WHERE sc.object_id = OBJECT_ID('CheckSumDemo')
ORDER BY sc.column_id;
GO

